#!/bin/bash
#

# if this build script complains that it cannot find bzlib.h, 
# you need to install bzip2-devel on Scientific Linux

usage()
{
   echo "USAGE: `basename ${0}` <product_dir> <e10|e14|e15|e17|e19|c2|c7> <debug|prof> [tar]"
}

# -------------------------------------------------------------------
# shared boilerplate
# -------------------------------------------------------------------

get_this_dir() 
{
    ( cd / ; /bin/pwd -P ) >/dev/null 2>&1
    if (( $? == 0 )); then
      pwd_P_arg="-P"
    fi
    reldir=`dirname ${0}`
    thisdir=`cd ${reldir} && /bin/pwd ${pwd_P_arg}`
}

get_ssibuildshims()
{
    # make sure we can use the setup alias
    if [ -z ${UPS_DIR} ]
    then
       echo "ERROR: please setup ups"
       exit 1
    fi
    source `${UPS_DIR}/bin/ups setup ${SETUP_UPS}`
    
    setup ssibuildshims ${ssibuildshims_version} -z ${product_dir}
}

# -------------------------------------------------------------------
# start processing
# -------------------------------------------------------------------

product_dir=${1}
basequal=${2}
extraqual=${3}
maketar=${4}

if [ -z ${product_dir} ]
then
   echo "ERROR: please specify the local product directory"
   usage
   exit 1
fi

# -------------------------------------------------------------------
# package name and version
# -------------------------------------------------------------------

package=vecgeom
origpkgver=v01_01_07
pkgver=${origpkgver}a
ssibuildshims_version=v1_04_14
pkgdotver=`echo ${origpkgver} | sed -e 's/_/./g' | sed -e 's/^v//'`
pkgtarfile=vecgeom-${pkgdotver}.tar

get_this_dir

get_ssibuildshims

source define_basics --

#.. ensure ${libdir} exists: ${pkgdir}/lib on Darwin, but ${pkgdir}/lib64 on Linux
libdir=${pkgdir}/lib
if [ $(uname) = "Linux" ]
then
    libdir=${pkgdir}/lib64
fi

if [ "${maketar}" = "tar" ] && [ -d ${libdir} ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
   exit 0
fi

echo "building ${package} for ${OS}-${plat}-${qualdir} (flavor ${flvr})"

# -------------------------------------------------------------------
# package dependent stuff goes here
# -------------------------------------------------------------------

case ${extraqual} in
debug) extra_command="-DCMAKE_BUILD_TYPE=Debug";;
prof)  extra_command="-DCMAKE_BUILD_TYPE=MinSizeRel";;
* )
   echo "ERROR: please specify debug, or prof"
   usage
   exit 1
;;
esac

mkdir -p ${pkgdir}
if [ ! -d ${pkgdir} ]
then
   echo "ERROR: failed to create ${pkgdir}"
   exit 1
fi
mkdir -p ${blddir}
if [ ! -d ${blddir} ]
then
   echo "ERROR: failed to create ${blddir}"
   exit 1
fi
mkdir -p ${sourcedir}
if [ ! -d ${sourcedir} ]
then
   echo "ERROR: failed to create ${sourcedir}"
   exit 1
fi

# declare now so we can setup
# fake ups declare
fakedb=${product_dir}/${package}/${pkgver}/fakedb
${SSIBUILDSHIMS_DIR}/bin/fake_declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

setup -B ${package} ${pkgver} -q ${fullqual} -z ${fakedb}:${product_dir}:${PRODUCTS} || ssi_die "ERROR: fake setup failed"

# We need cmake
source ${SSIBUILDSHIMS_DIR}/bin/setup_cmake || ssi_die "ERROR: failed to setup camke"

if [[ "${basequal}" == e1[0245] ]]
then
  cc=gcc
  cxx=g++
  extra_command="${extra_command} -DCMAKE_CXX_STANDARD=14"
elif [[ "${basequal}" == e1[79] ]]
then
  cc=gcc
  cxx=g++
  extra_command="${extra_command} -DCMAKE_CXX_STANDARD=17"
elif [[ "${basequal}" == c[27] ]]
then
  cc=clang
  cxx=clang++
  extra_command="${extra_command}  -DCMAKE_CXX_FLAGS=-std=c++17 -DCMAKE_CXX_STANDARD=17"
else
  ssi_die "Qualifier $basequal not recognized."
fi

ncore=`${SSIBUILDSHIMS_DIR}/bin/ncores`

set -x
cd ${sourcedir}
tar xf ${tardir}/${pkgtarfile}
# apply patch
patchname=${package}-${pkgdotver}.patch
if [[ -f "${patchdir}/${patchname}" ]]; then
  patch -b -p0 < "${patchdir}/${patchname}" || ssi_die "ERROR: application of patch ${patchdir}/${patchname} failed"
fi

# copy test files 
cp -p ${patchdir}/cms2015.root ${sourcedir}/${package}-${pkgdotver}/test/cmstestdata/
cp -p ${patchdir}/ExN03.root ${blddir}/

cd ${blddir}

env CC=${cc} CXX=${cxx} FC=gfortran cmake \
          -DCMAKE_INSTALL_PREFIX=${pkgdir} \
	  -DBACKEND=Scalar -DGEANT4=OFF \
          -DUSOLIDS=ON \
          -DVECGEOM_VECTOR=sse3 \
	  ${extra_command} \
	  ${sourcedir}/${package}-${pkgdotver}
(( $? == 0 )) || ssi_die "ERROR: cmake failed."
make -j ${ncore} || ssi_die "ERROR: make failed."
make test || ssi_die "ERROR: make test failed."
make install  || ssi_die "ERROR: make install failed."

set +x

if [ ! -d ${libdir} ]
then
   echo "ERROR: failed to create ${libdir}"
   echo "       Something is very wrong"
   exit 1
fi

# real ups declare
${SSIBUILDSHIMS_DIR}/bin/declare_product ${product_dir} ${package} ${pkgver} ${fullqual}

# -------------------------------------------------------------------
# common bottom stuff
# -------------------------------------------------------------------

# this should not complain
echo "Finished building ${package} ${pkgver}"
setup ${package} ${pkgver} -q ${fullqual} -z ${product_dir}:${PRODUCTS}
echo "vecgeom is installed at ${VECGEOM_DIR}"

# fix paths for cmake config
cmakeConfig=`ls ${VECGEOM_LIB}/cmake/*/*.cmake`
for file in ${cmakeConfig}
do
  mv ${file} ${file}.bak
  if [[ "${basequal}" == c[27] ]]; then
      cat ${file}.bak | sed -e "s%${VECGEOM_FQ_DIR}%\$ENV{VECGEOM_FQ_DIR}%g" \
	  | sed -e "s%${VECCORE_FQ_DIR}%\$ENV{VECCORE_FQ_DIR}%g" \
	  | sed -e "s%${CLANG_FQ_DIR}%\$ENV{CLANG_FQ_DIR}%g" \
		> ${file}
  else
      cat ${file}.bak | sed -e "s%${VECGEOM_FQ_DIR}%\$ENV{VECGEOM_FQ_DIR}%g" \
	  | sed -e "s%${VECCORE_FQ_DIR}%\$ENV{VECCORE_FQ_DIR}%g" \
	  | sed -e "s%${GCC_FQ_DIR}%\$ENV{GCC_FQ_DIR}%g" \
		> ${file}
  fi
done


# this must be last
if [ "${maketar}" = "tar" ] && [ -d ${libdir} ]
then
   ${SSIBUILDSHIMS_DIR}/bin/make_distribution_tarball ${product_dir} ${package} ${pkgver} ${fullqual}
fi

exit 0
